# FILE: bug83.tcl
#
#	Patches for Tcl 8.3
#
# Last modified on: $Date: 2006-06-07 20:35:32 $
# Last modified by: $Author: dgp $
#
# This file contains Tcl code which when sourced in a Tcl 8.3 interpreter
# will patch bugs in Tcl 8.3 fixed in later versions of Tcl.  It is not
# meant to be complete.  Bugs are patched here only as we discover the need
# to do so in writing other OOMMF software.

# Before Tcl 8.4, [expr rand()] can return an out of range result
# the first time it is called on a 64-bit platform.  Workaround that
# problem by always calling it and discarding the possibly bad result
# here before good results are expected.  Use [catch] to cover Tcl 7
# where rand() did not yet exist.
catch {expr {rand()}}

# Some new options were added to [lsearch] in 8.4 that are handy to
# make use of.

rename lsearch Tcl8.3_lsearch
;proc lsearch {args} {
    if {[llength $args] < 2} {
	return [uplevel 1 Tcl8.3_lsearch $args]
    }
    set pattern [lindex $args end]
    set list [lindex $args end-1]
    set options [lrange $args 0 end-2]

    set useAll 0
    set legalOptions {}
    foreach o $options {
	switch -exact -- $o {
	    -ascii	-
	    -decreasing	-
	    -dictionary	-
	    -increasing	-
	    -inline	-
	    -integer	-
	    -not	-
	    -real	-
	    -sorted 	-
	    -start	{
		return -code error "\[lsearch] option \"$o\" not supported;\
			Upgrade to Tcl 8.4 or higher."
	    }
	    -all {
		set useAll 1
	    }
	    default {
		lappend legalOptions $o
	    }
	}
    }
    if {!$useAll} {
	return [uplevel 1 Tcl8.3_lsearch $options [list $list $pattern]]
    }
    set result {}
    set idx [uplevel 1 Tcl8.3_lsearch $legalOptions [list $list $pattern]]
    set offset 0
    while {$idx >= 0} {
	incr offset $idx
	lappend result $offset
	incr offset
	set list [lrange $list [incr idx] end]
	set idx [uplevel 1 Tcl8.3_lsearch $legalOptions [list $list $pattern]]
    }
    return $result
}

# Tcl 8.3 is the current stable version.
# Continue on to the generic patches.
source [file join [file dirname [info script]] bug.tcl]

